#ifndef __IUF_TRIANGLE
#define __IUF_TRIANGLE
#include <math.h>
#include <vcg/Point3.h>
#include <vector>
using namespace std;
using namespace vcg;

/** Trivial class to represent a triangle with its normal.
For each triangle we store three point3f and a normal. 
Note that this is a very naive approach, we do not take into account
that vertexes can be shared or that normal can be {\em per vertex}
instead of {\em per-face}.
*/
class Triangle
{
public:
/// The three vertexes
 Point3f v[3];
/// The Normal of the face 
 Point3f N;

/** Compute the normal of the face starting from the three vertexes
 and store it in the \Ref{N} member */
 void CalcNormal();

/// Less operator needed for stl containers
 inline bool operator <  ( Triangle const & p ) const {
			return	(v[2]!=p.v[2])?(v[2]<p.v[2]):
							(v[1]!=p.v[1])?(v[1]<p.v[1]):
							(v[0]<p.v[0]);
	}
/// Equal operator needed for some stl containers
 inline bool operator == ( Triangle const & p ) const {
			return (v[0]==p.v[0] && v[1]==p.v[1] && v[2]==p.v[2]);
	} 

};

/** Trivial class to represent a simple triangular mesh.
This class will be strongly updated in the next releases 
to include some more sophisticated(!) features like colors 
normal and textures.
*/
class Mesh
{
public:
	/** All the triangles are stored in a STL vector. 
	@name vector<\Ref{Triangle}> T*/
	vector<Triangle> T;
	
	/** Simply Draw all the Triangles stored in \Ref{T};*/
	void Draw();
	
	/** Load a mesh from a file.
	The format is the so-called {\em RAW} ascii format, for each triangle
	just the ascii print of xyz coordinates of its vertexes. 
	E.g. a tetrahedron is stored as:
	\begin{verbatim}
	 1  1  1    -1 -1  1     1 -1 -1
	-1 -1  1     1  1  1    -1  1 -1
	 1 -1 -1    -1 -1  1    -1  1 -1
	-1  1 -1     1 -1 -1     1  1  1
	\end{verbatim}
*/
	void Open(char *filename);
};
#endif